/* -*- mode: c++; c-basic-offset: 3; -*- */
#include "multiframe_writer.hh"
#include <stdexcept>

using namespace std;

//======================================  Constructor
multiframe_writer::multiframe_writer(const std::string& name, int run,
				     Interval length)
   : FrWriter(name, run), mLength(length), mAligned(false), mRename(false)
{}

//======================================  Destructor
multiframe_writer::~multiframe_writer(void) {
   if (haveFrame()) write();
   if (is_open()) close();
}

//======================================  close the frame file.
void
multiframe_writer::close(void) {
   if (!is_open()) return;
   FrWriter::close();
   Interval dT = mFileNow - mFileStart;
   string path = mFrameName.file_path(mFileStart, dT);
   if (path != getFile()) {
      mFrameName.move_to_path(getFile(), mFileStart, dT);
   }
   mFileStart = Time(0);
}

//======================================  calculate end time.
Time
multiframe_writer::endTime(void) const {
   if (!mFileStart || mLength <= Interval(0)) return mFileNow;
   Time endt(mFileStart + mLength);
   if (mAligned) endt -= Interval(fmod(mFileStart.totalS(), double(mLength)));
   return endt;
}

//======================================  open a frame output stream
void
multiframe_writer::open(void) {
   if (is_open() || !haveFrame()) return;
   mFileStart = getTime();
   mFileNow   = mFileStart + getFrLength();
   Interval dT = endTime() - mFileStart;

   //-----------------------------------  Get a path and make sure the
   //                                     directory exists.
   string path;
   if (!mFrameName.online()) {
      if (mRename) path = mFrameName.temp_path(mFileStart, dT);
      else         path = mFrameName.file_path(mFileStart, dT);
      mFrameName.make_dir(mFrameName.dir_name(mFileStart));
   } else {
      path = path = mFrameName.file_path(mFileStart, dT);
   }

   //-----------------------------------  open the frame file.
   FrWriter::open(path, true);
}

//======================================  Set the output file name template
void
multiframe_writer::set_file_name(const std::string& name) {
   mFrameName.split(name);
}

//======================================  Set the output file name template
void
multiframe_writer::set_file_name(const string& dir, const string& prefix,
				 const string& ext) {
   mFrameName.set_directory(dir);
   mFrameName.set_prefix(prefix);
   mFrameName.set_extension(ext);
   mFrameName.set_dt(0);
}

//======================================  Set the output file length in seconds
void
multiframe_writer::set_file_length(Interval length) {
   mLength = length;
   mAligned = false;
}

//======================================  Set the file length alignment
void
multiframe_writer::set_file_alignment(Interval length) {
   mLength = length;
   mAligned = true;
}

//======================================  Set the rename optuion
void
multiframe_writer::set_rename(bool yn) {
   mRename = yn;
}

//======================================  Write out a frame
void
multiframe_writer::write(void) {
   if (!haveFrame()) return;
   
   //-----------------------------------  Check we can use open file
   Time t0   = getTime();
   if (is_open() && t0 != mFileNow) {
      if (t0 < mFileNow) {
	 cerr << "multiframe_writer: New frame time: " << t0.getS()
	      << " overlaps previous end-time: " << mFileNow.getS() << endl;
	 throw runtime_error("multiframe_writer: Overlapping frames in file");
      }
      close();
   }

   //-----------------------------------  Open if appropriate
   if (!is_open()) {
      open();
   } else {
      mFileNow = t0 +  getFrLength();
   }

   //-----------------------------------  write and erase the frame
   writeFrame();

   //-----------------------------------  See if this is the end of the file
   if (mFileNow >= endTime()) {
      close();
   }
}

